/*
	DaisyChain IRQ handler
*/
#include <windows.h>
#include <stdio.h>

#include "x1.h"
#include "x1_irq.h"

/* higher request device, and assert device */
static BYTE irq_request;
static BYTE irq_ieolock;

static BYTE irq_int[MAX_DAISY_CHAIN+1];
static BYTE irq_ieo[MAX_DAISY_CHAIN+1];
static BYTE irq_vector[MAX_DAISY_CHAIN+1];

/* check IRQ and assert IRQ */
void __fastcall X1_check_irq_line(void)
{
	if( (irq_request > irq_ieolock) && Z80_Able_interrupt() )
	{	/* enter interrupt */
		irq_ieolock = irq_request;
		irq_int[irq_request] = 0;
		if( irq_request != DEVICE_SUB) //SUB CPU͂ԂIEOĂȂ
			irq_ieo[irq_request] = 1;
		/* search next IRQ device */
		while(--irq_request > 0)
		{
			if(irq_int[irq_request])
				break;
		}
		Z80_Cause_Interrupt2( irq_vector[irq_ieolock] );
	}
	/* for Z80emu */
	__asm{ and  ecx,0x0000ffff };
}

/* clear IEO lock */
void __fastcall X1_irq_reti(void)
{
	BYTE reti_device = irq_ieolock;

	if(reti_device)
	{
		/* clear higher IEO */
		irq_ieo[reti_device] = 0;

		/* search next IEO device */
		while(--irq_ieolock > 0)
		{
			if(irq_ieo[irq_ieolock])
			{
				X1_check_irq_line();
				break;
			}
		}
		/* for Z80DMA , enable after RETI */
//		if(reti_device == DEVICE_DMA)
//			dma_reti();
	}
}

void __fastcall X1_set_irq_line(BYTE device)
{
	irq_int[device] = 1;

	if(irq_request < device)
	{
		irq_request = device;
		X1_check_irq_line();
	}
}

void __fastcall X1_clear_irq_line(BYTE device)
{
	irq_int[device] = 0;

	if(irq_request == device)
	{
		/* search next IRQ device */
		while(--irq_request >0)
		{
			if(irq_int[irq_request])
				break;
		}
	}
}

void __fastcall X1_clear_ieo_line(BYTE device)
{
	irq_ieo[device] = 0;

	if(irq_ieolock == device)
	{
		/* search next IEO device */
		while(--irq_ieolock >0)
		{
			if(irq_ieo[irq_ieolock])
				break;
		}
		X1_check_irq_line();
	}
}

BYTE __fastcall X1_check_pending_irq(BYTE device)
{
	return irq_int[device] | irq_ieo[device];
}

void __fastcall X1_set_irq_vector(BYTE device,BYTE vector)
{
	irq_vector[device] = vector;
}

void init_irq(void)
{
	int i;

	irq_request = irq_ieolock = 0;
	for(i=0; i<=MAX_DAISY_CHAIN; i++)
		irq_int[i]= irq_ieo[i]= irq_vector[i]= 0;
}
